clear; close all;

%% instructions 
% ######################################################
% ###   this code solves the QG omega equation       ###
% ######################################################
% the QG omega equation can refer to Holton 2004 or Eq.1 in Dai and Nie (2020)
% 
% ============  Input data =============
% lon:    [lon], longitude
% lat:    [lat], latitude (from small to big)
%         **Only the northern or southern hemisphere can be calculated at a time**
% level:  [level], pressure level (from small to big) unit: hPa
% time:   [time], **at least input 3 time slices**
%         time=(1:nt)/(24/time_interval);  "nt" is the total number of days, the unit of "time_interval" is hour
%         e.g., for daily data, time_interval=24 hour, time=1:nt;
%               for 6-hour data, time_interval=6 hour, time=(1:nt)/4.0;
%
% z:      [lon,lat,level,time], geopotential, unit: m2/s2, 
% T:      [lon,lat,level,time], air temperature, unit: K
% w:      [lon,lat,level,time], pressure vertical velocity, unit: Pa/s
% sp:     [lon,lat,time], surface pressure, unit: Pa
%
% ============  Output/Save data =============
% ## save qgw in a ".nc" file ##
% qgw:    [lon,lat,level,4,time], unit:Pa/s
%         the forth dimension is the four terms of qgw: 
%           1: w forced by vorticity advection, 
%           2: w forced by temperature advection, 
%           3: w forced by diabatic heating, 
%           4: w forced by boundary forcing
% Forcing:[lon,lat,level,3,time]
%         the forth dimension is the three rhs terms of QGw equation: 
%           1: vorticity advection, 
%           2: temperature advection, 
%           3: diabatic heating

%% Input: 
% ##### need to change the following inputs !!!! #####
readpath=pwd;       % data path
addpath(genpath([readpath,'\codes\'])); % Add the file "codes1" to MATLAB path.
savename_qgw=[readpath,'\qgw.nc']; 
savename_F=[readpath,'\Forcing.nc']; 
savename_fig=[readpath,'\fig1']; 


load([readpath,'/ERA_19790101_0103.mat'])
% z  %geopotential m2/s2
% T 
% w   %Pa/s
% sp  %Pa
% lon  
% lat   
% level  %hPa
[nlon,nlat,nlev,nt]=size(z);
time  = (1:nt);

% ########################################################
%                   end of input !!! 
% ########################################################

%% ========== calculate forcing and qgw ==========
% basic calculation
level=double(level);
dens=calcDens(T,level);
[sigma,sigmaLoc]=calcSigma(T,level);
sigmaLoc(sigmaLoc<0)=0;
f=2*7.2921e-5*sin(lat*pi/180);  
[uGeostr,vGeostr]=calcGeostr(z,lon,lat,f); 

fprintf('------- calculating RHS terms...... -------\n');
[firstTerm,secondTerm,thirdTerm]=calcTerms(z,uGeostr,vGeostr,w,sigmaLoc,lon,lat,level,time,f);
Forcing=zeros(nlon,nlat,nlev,3,nt);
Forcing(:,:,:,1,:)=firstTerm;
Forcing(:,:,:,2,:)=secondTerm;
Forcing(:,:,:,3,:)=thirdTerm;

fprintf('------- inversion of QGw...... -------\n');
[qgw,ifail]=calcOmega(firstTerm,secondTerm,thirdTerm,lon,lat,level,sigmaLoc,sp,dens,w,f);
qgw=permute(qgw,[1,2,3,5,4]); %change time to the last dimension

%% ========== save ==========
save_nc(savename_qgw,qgw,lon,lat,level,'qgw');
save_nc(savename_F,Forcing,lon,lat,level,'Forcing');

% data=zeros(nlon,nlat,nlev,5,nt);
% data(:,:,:,1:4,:)=qgw;
% data(:,:,:,5,:)=w;
% save_nc([readpath,'\data.nc'],data,lon,lat,level,'data');

%% ========== plot ==========
% qgw inversion terms
w_vor=qgw(:,:,:,1,2); 
w_T=qgw(:,:,:,2,2);
w_Q=qgw(:,:,:,3,2); 
w_BF=qgw(:,:,:,4,2);
w_qg=w_vor+w_T+w_Q+w_BF;
w=w(:,:,:,2);

figure('position',[1,1,1200,1200]);
 % 500hPa level
subplot(3,2,1) 
contourf(lon,lat,w_vor(:,:,level==500)',[-0.3 -0.25 -0.2 -0.15 -0.1 -0.05 0 0.05 0.1 0.15 0.2 0.25 0.3]); colorbar; caxis([-0.3 0.3]);
title('\w_vor')
subplot(3,2,2) 
contourf(lon,lat,w_T(:,:,level==500)',[-0.3 -0.25 -0.2 -0.15 -0.1 -0.05 0 0.05 0.1 0.15 0.2 0.25 0.3]); colorbar; caxis([-0.3 0.3]);
title('\w_T')
subplot(3,2,3) 
contourf(lon,lat,w_Q(:,:,level==500)',[-0.3 -0.25 -0.2 -0.15 -0.1 -0.05 0 0.05 0.1 0.15 0.2 0.25 0.3]); colorbar; caxis([-0.3 0.3]);
title('\w_Q')
subplot(3,2,4) 
contourf(lon,lat,w_BF(:,:,level==500)',[-0.3 -0.25 -0.2 -0.15 -0.1 -0.05 0 0.05 0.1 0.15 0.2 0.25 0.3]); colorbar; caxis([-0.3 0.3]);
title('\w_BF')
subplot(3,2,5) 
contourf(lon,lat,w_qg(:,:,level==500)',[-0.6 -0.5 -0.4 -0.3 -0.2 -0.1 0 0.1 0.2 0.3 0.4 0.5 0.6]); colorbar; caxis([-0.6 0.6]);
title('\w_qg')
subplot(3,2,6) 
contourf(lon,lat,w(:,:,level==500)',[-0.6 -0.5 -0.4 -0.3 -0.2 -0.1 0 0.1 0.2 0.3 0.4 0.5 0.6]); colorbar; caxis([-0.6 0.6]);
title('\w_ERA')


colormap('hsv');
print(gcf,'-dpng','-r300',savename_fig)








