function R = ellipticPDEsolver(IT,R,NX,NY,NZ,AA,BB,CC,DD,EE,FF,GG)
% DShaevitz 11/2014 - Based on fortran code from ETH
% R = ellipticPDEsolver(IT,R,NX,NY,NZ,AA,BB,CC,DD,EE,FF,GG)
% Strongly implicit method of solving PDE - STONE H.L., 1968, SIAM J.
%    Num. Anal., 5, p.530
% Algorithm based on the NAG routine D03UBF


% Cycle parameter
ALP=[1.0,0.625,0.25,0.875,0.5,0.125,0.75,0.375,0.0];

NY1=NY+1;
NZ1=NZ+1;

% Set odd/even counter  ODD_OR_EVEN=1  for odd iterations
%                       ODD_OR_EVEN=2  for even iterations
ODD_OR_EVEN = mod(IT-1,2) + 1;
if ODD_OR_EVEN == 0
    ODD_OR_EVEN = 2;
end
ODD_EVEN    = 2 - ODD_OR_EVEN;
DODD_EVEN   = double(ODD_EVEN);
EVEN_ODD    = ODD_OR_EVEN - 1;
DEVEN_ODD   = double(EVEN_ODD);
    
% Determine the number of the acceleration parameter to
%  be used, the same parameter is used for 2 iterations.
%  The iteration parameter STONE eq (27)[
IS    = floor((mod(IT-1,18))/2 + 1);
ALM   = 45.0/double((NX-1)*(NX-1)+(NY-1)*(NY-1)+(NZ-1)*(NZ-1));
ALPHA = 1.0 - ALM^ALP(IS);
    
for KK = 1:NZ

% KK is only a counter in the third coordinate direction
% K  is the index of the third coordinate and on alternate
%     iterations scans first increasing the decreasing
   K  = ODD_EVEN*KK + EVEN_ODD*(NZ1-KK);
   KB = K - ODD_EVEN + EVEN_ODD;

   for JJ = 1:NY

% JJ is only a counter in the second coordinate direction
%  J is the index of the third coordinate and on alternate
%     iterations scans first increasing then decreasing
      J  = ODD_EVEN*JJ + EVEN_ODD*(NY1-JJ);
      JB = J - ODD_EVEN + EVEN_ODD;
      
      
      for I = 1:NX

         IB  = I - 1;
         AAA = AA(I,J,K);
         BBB = BB(I,J,K);
         CCC = CC(I,J,K);
         DDD = DD(I,J,K);
         EEE = EE(I,J,K);
         FFF = FF(I,J,K);
         GGG = GG(I,J,K);
         if DDD ~= 0.0

% Seven point molecule equation
            if ((KB ~= 0) && (KB ~= NZ1))
               S3  = W1(I,J,KB);
               S6  = W2(I,J,KB);
               S9  = W3(I,J,KB);
               R03 = R(I,J,KB);
            else
               S3  = 0.0;
               S6  = 0.0;
               S9  = 0.0;
               R03 = 0.0;
            end      

            if ((JB ~= 0) && (JB ~= NY1))
               S2  = W1(I,JB,K);
               S5  = W2(I,JB,K);
               S8  = W3(I,JB,K);
               R02 = R(I,JB,K);
            else
               S2  = 0.0;
               S5  = 0.0;
               S8  = 0.0;
               R02 = 0.0;
            end

            if (I ~= 1)
               S1  = W1(IB,J,K);
               S4  = W2(IB,J,K);
               S7  = W3(IB,J,K);
               R01 = R(IB,J,K);
            else
               S1  = 0.0;
               S4  = 0.0;
               S7  = 0.0;
               R01 = 0.0;
            end      

% Calculate the elements of the lower triangular matrix L
            SA = (DODD_EVEN*AAA+DEVEN_ODD*GGG)/(1.0+ALPHA*(S3+S6));
            SB = (DODD_EVEN*BBB+DEVEN_ODD*FFF)/(1.0+ALPHA*(S8+S2));
            SC = CCC/(1.0+ALPHA*(S7+S4));

            E1 = SA*S3 + SB*S2;
            E2 = SC*S4 + SA*S6;
            E3 = SB*S8 + SC*S7;           
            
% Calculate the value of the diagonal element of L
            SD = 1.0/(DDD+ALPHA*(E1+E2+E3)-(SC*S1+SB*S5+SA*S9));

% Calculate and store the elements of the upper triangular matrix
            W1(I,J,K) = (EEE-ALPHA*E1)*SD;
            W2(I,J,K) = (DODD_EVEN*FFF+DEVEN_ODD*BBB-ALPHA*E2)*SD;
            W3(I,J,K) = (DODD_EVEN*GGG+DEVEN_ODD*AAA-ALPHA*E3)*SD;

% Calculation of the elements of the inverse of L * RES vector
            R(I,J,K) = SD*(R(I,J,K)-SA*R03-SB*R02-SC*R01);

         else
% Calculation for the explicit equation
            W1(I,J,K) = 0.0;
            W2(I,J,K) = 0.0;
            W3(I,J,K) = 0.0;
         end

      end

   end

end

% Progress backwards to multiply by the inverse of the matrix U
%^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
for KK = 1:NZ

   K  = ODD_EVEN*(NZ1-KK) + EVEN_ODD*KK;
   KB = K + ODD_EVEN - EVEN_ODD;

   for JJ = 1:NY

      J  = ODD_EVEN*(NY1-JJ) + EVEN_ODD*JJ;
      JB = J + ODD_EVEN - EVEN_ODD;

      for I = NX:-1:1

         if ((KB ~= 0) && (KB ~= NZ1))
              R(I,J,K) = R(I,J,K) - W3(I,J,K)*R(I,J,KB);
         end
         if ((JB ~= 0) && (JB ~= NY1)) 
              R(I,J,K) = R(I,J,K) - W2(I,J,K)*R(I,JB,K);
         end
         if (I ~= NX) 
             R(I,J,K) = R(I,J,K) - W1(I,J,K)*R(I+1,J,K);
         end

      end

   end

end
%^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
% KK = 1:NZ;
% K  = ODD_EVEN*(NZ1-KK) + EVEN_ODD*KK;
% KB = K + ODD_EVEN - EVEN_ODD;
% 
% id_KB=(KB ~= 0) & (KB ~= NZ1);
% R(:,:,K(id_KB)) = R(:,:,K(id_KB)) - W3(:,:,K(id_KB)).*R(:,:,KB(id_KB));
% 
% JJ = 1:NY;
% J  = ODD_EVEN*(NY1-JJ) + EVEN_ODD*JJ;
% JB = J + ODD_EVEN - EVEN_ODD;
% 
% id_JB=(JB ~= 0) & (JB ~= NY1);
% R(:,J(id_JB),:) = R(:,J(id_JB),:) - W2(:,J(id_JB),:).*R(:,JB(id_JB),:);
% 
% for I=NX-1:-1:1;
%     R(I,:,:) = R(I,:,:) - W1(I,:,:).*R(I+1,:,:);
% end
% 
% end
%^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
