function [omega,w,ifail] = invertOmega(RHS,lon,lat,level,sigma,f0,surfBC,surfP,dens)
%
% DShaevitz 11/2014 - Based on fortran code from ETH
% Invert omega for 1 timestep
% [omega,w,ifail] = invertOmega(RHS,lon,lat,level,sigma,f0,surfBC,surfP,dens)
%
% Del^2 omega = RHS
%
% Inputs: RHS(lon,lat,level)      Right hand side of omega equation
%         lon                     Longitude in degrees
%         lat                     Latitude in degrees 
%         level                   Pressure in mb
%         sigma(level)            Static stability   %----- change by panxi:use sigma(lon,lat,level)   
%         f0                      f0                 %----- change by panxi:use f0(lat) instead of constant f0
%         surfBC(lon,lat)         Surface Boundary Condition for omega
%         surfP(lon,lat)          Surface Pressure in Pa
%         dens(lon,lat,level)     Density
%
% Outputs: omega(lon,lat,level)   QG Omega
%          w(lon,lat,level)       QG w
%          ifail                  Equal to 1 if solution failed to converge
%

pres=level*100.;
nx=length(lon);
ny=length(lat);
nz=length(level);

AA=zeros(size(RHS));
BB=AA;
CC=AA;
DD=AA;
EE=AA;
FF=AA;
GG=AA;

% %^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
% for k=2:nz-1
%     
%     deltapP=pres(k+1)-pres(k);
%     deltapM=pres(k)-pres(k-1);
%     deltapT=pres(k+1)-pres(k-1);
%     
%     for i=2:nx-1
%         for j=2:ny-1
%             
% %             cosphi=cos(lat(j)*pi/180);
%             
%             AA(i,j,k)=f0(j)^2/deltapM/deltapT*2;
%             GG(i,j,k)=f0(j)^2/deltapP/deltapT*2;
%         
%             distX=lldistkm([lat(j) lon(i+1)],[lat(j) lon(i-1)])*1000/2;
%             distY=lldistkm([lat(j+1) lon(i)],[lat(j-1) lon(i)])*1000/2;
%             
%             % ---- change by panxi ------
%             %BB(i,j,k)=sigma(k)/distY^2;
%             %CC(i,j,k)=sigma(k)/distX^2;      
%             BB(i,j,k)=sigma(i,j,k)/distY^2;
%             CC(i,j,k)=sigma(i,j,k)/distX^2;
%             % ---------------------------
%         end
%     end
% end
% %^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

%^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

pres1=permute(repmat(pres,[1,nx,ny]),[2,3,1]);
pres1=pres1(2:nx-1,2:ny-1,:);
deltapP=pres1(:,:,3:nz)-pres1(:,:,2:nz-1);   % (k+1)-k
deltapM=pres1(:,:,2:nz-1)-pres1(:,:,1:nz-2); % k-(k-1)
deltapT=pres1(:,:,3:nz)-pres1(:,:,1:nz-2);   % (k+1)-(k-1)
%
f1=permute(repmat(f0,[1,nx,nz]),[2,1,3]);
f1=f1(2:nx-1,2:ny-1,2:nz-1);

AA(2:nx-1,2:ny-1,2:nz-1)=f1.^2./deltapM./deltapT*2;
GG(2:nx-1,2:ny-1,2:nz-1)=f1.^2./deltapP./deltapT*2;
%distX
distX=zeros(ny,1);
for j=2:ny-1
    distX(j)=lldistkm([lat(j) lon(3)],[lat(j) lon(1)])*1000/2;
end
distX=permute(repmat(distX,[1,nx,nz]),[2,1,3]);
%dy
distY=lldistkm([lat(3) lon(1)],[lat(1) lon(1)])*1000/2; 
BB(2:nx-1,2:ny-1,2:nz-1)=sigma(2:nx-1,2:ny-1,2:nz-1)/distY^2;
CC(2:nx-1,2:ny-1,2:nz-1)=sigma(2:nx-1,2:ny-1,2:nz-1)./(distX(2:nx-1,2:ny-1,2:nz-1).^2);
%^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^


FF=BB;
EE=CC;
DD=-(AA + BB + CC + EE + FF + GG);

%
% Invert laplacian to compute QG omega
%
[ omega, ifail ] = invlap3d(RHS,nx,ny,nz,surfBC,surfP,level,AA,BB,CC,DD,EE,FF,GG);
 
%
% Convert QG omega to QG w
%
w=-omega./dens/9.8;            

end

